/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/21 17:01:37 $
 // $Revision: 1.7 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/


#include "cdcover.h"

using namespace std;

/**
 * Constructors/Destructors
 */
/**
 * Constructor with in- and outfile, much more useful for daily use
 * @param infile file to pass to Infofactory to get track information
 * @param outfile name of hte PS-document on the disk
 */
CDCover::CDCover (CDInfo info, string infile, string outfile){
	CDInfoFactory* fact = new CDInfoFactory(info,infile);
	content = fact->get_content();

	/**
	* Initialize the PS document
	*/
	PS_boot();		// sets locale 
	sheet = PS_new();
	PS_open_file(sheet,outfile.c_str());
	PS_set_info(sheet,"Creator","cdinlay");
	PS_set_info(sheet,"Author","Thomas Ruschival");
	PS_set_info(sheet,"Title",outfile.c_str());
	loadfonts();	// do the more complex action of loading fonts
	PS_begin_page(sheet,593.2,838.9);
	// Scale coordinates to "normal" metric mm-scale instead of imperial-weirdness 1/72dpi
	PS_scale(sheet,2.825,2.825);
	PS_translate(sheet,15.0,160.0);
}
    
/**
 * Methods
 */
/**
 * Get the value of content
 * All information that will be displayed on the CD cover
 * @return the value of content
 */
CDInfo CDCover::get_content ( ) {
	return content;
}
/**
 * Set the value of content
 * All information that will be displayed on the CD cover
 * @param value the value of content
 */
void CDCover::set_content (CDInfo& value ) {
	content=value;
}
/**
 * Get the value of back
 * backinlay, currently the only thing we want to print
 * @return the value of back
 */
BackInlay* CDCover::get_back ( ) {
	return back;
}
/**
 * Set the value of back
 * backinlay, currently the only thing we want to print
 * @param value the value of back
 */
void CDCover::set_back (BackInlay* value ) {
	back=value;
}
/**
 * Get the value of sheet
 * PDFhandle to write on
 * @return the value of sheet
 */
PSDoc* CDCover::get_sheet ( ) {
	return sheet;
}
/**
 * Set the value of sheet
 * PDFhandle to write on
 * @param value the value of sheet
 */
void CDCover::set_sheet (PSDoc* value ) {
	sheet=value;
}
/**
 * Get the value of fonts
 * Fonts used on this PS-document. pslib gives each font it generated a specific number
 * @return the value of fonts
 */
FontSet  CDCover::get_fonts ( ) {
	return fonts;
}
/**
 * Set the value of fonts
 * Fonts used on this PS-document. pslib gives each font it generated a specific number
 * @param value the value of fonts
 */
void CDCover::set_fonts (FontSet  value ) {
	fonts=value;
}
/**
 * produce a PS Document with a Backinlay onto 
 */
void CDCover::generate (){
	back = new BackInlay(sheet,&fonts,content);
	PS_end_page(sheet);
	PS_shutdown();	
}
/**
 * Create a fontset for this PS-Document
 */
void CDCover::loadfonts () {
	fonts.artistfont.psfont_handle=PS_findfont(sheet,ARTISTFONT,"",0);
	fonts.artistfont.size=ARTISTFONTSIZE;
	fonts.titlefont.psfont_handle=PS_findfont(sheet,TITLEFONT,"",0);
	fonts.titlefont.size=TITLEFONTSIZE;
	fonts.subtitlefont.psfont_handle=PS_findfont(sheet,SUBTITLEFONT,"",0);
	fonts.subtitlefont.size=SUBTITLEFONTSIZE;
	fonts.trackfont.psfont_handle=PS_findfont(sheet,TRACKFONT,"",0);
	fonts.trackfont.size=TRACKFONTSIZE;
	fonts.spinefont.psfont_handle=PS_findfont(sheet,SPINEFONT,"",0);
	fonts.spinefont.size=SPINEFONTSIZE;
	fonts.genericfont.psfont_handle=PS_findfont(sheet,GENERICFONT,"",0);
	fonts.genericfont.size=GENERICFONTSIZE;
	fonts.titlefont_small.psfont_handle=PS_findfont(sheet,TITLEFONT,"",0);
	fonts.titlefont_small.size=TRACKFONTSIZE;
	fonts.artistfont_small.psfont_handle=PS_findfont(sheet,ARTISTFONT,"",0);
	fonts.artistfont_small.size=TRACKFONTSIZE;
}
	

