/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/14 15:58:57 $
 // $Revision: 1.4 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/


#ifndef CDCOVER_H
#define CDCOVER_H
#include <string>

extern "C"{	
	#include <libps/pslib.h>		// used for generation of Postscript documents
}

#include "backinlay.h"
#include "cdinfofactory.h"
#include "cdinfo.h"
#include "cdinlay_types.h"

using namespace std;

/**
 * Class CDCover
 * Holds everything together, at the moment it just controls
 * the reading of CDinfo and writing the Backinlay. 
 * Supposed to use different CDInfoReaders for different cases
 */
class CDCover {
	/**
	 * Public stuff
	 */
 public:
	/**
	 * Constructors
	 */
	/**
	 * Constructor with in- and outfile, much more useful for daily use
	 * @param infile file to pass to Infofactory to get track information
	 * @param outfile name of hte PS-document on the disk
	 */
	CDCover (CDInfo info, string infile, string outfile);

	
	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of content
	 * All information that will be displayed on the CD cover
	 * @return the value of content
	 */
	CDInfo get_content ( );
		
	
	/**
	 * Set the value of content
	 * All information that will be displayed on the CD cover
	 * @param value the value of content
	 */
	void set_content (CDInfo& value );
		
	
	/**
	 * Get the value of back
	 * backinlay, currently the only thing we want to print
	 * @return the value of back
	 */
	BackInlay* get_back ( );
		
	
	/**
	 * Set the value of back
	 * backinlay, currently the only thing we want to print
	 * @param value the value of back
	 */
	void set_back (BackInlay* value );
		
	
	/**
	 * Get the value of sheet
	 * PDFhandle to write on
	 * @return the value of sheet
	 */
	PSDoc* get_sheet ( );
		
	
	/**
	 * Set the value of sheet
	 * PDFhandle to write on
	 * @param value the value of sheet
	 */
	void set_sheet (PSDoc* value );
		
	
	/**
	 * Get the value of fonts
	 * Fonts used on this PS-document. pslib gives each font it generated a specific number
	 * @return the value of fonts
	 */
	FontSet  get_fonts ( );
		
	
	/**
	 * Set the value of fonts
	 * Fonts used on this PS-document. pslib gives each font it generated a specific number
	 * @param value the value of fonts
	 */
	void set_fonts (FontSet  value );
		
	
	/**
	 * Operations
	 */

	/**
	 * Operations
	 */
	/**
	 * produce a PS Document with a Backinlay onto 
	 */
	void  generate ();

	/**
	 * Private stuff
	 */
 private:
	/**
	 * Fields
	 */
	/**
	 * All information that will be displayed on the CD cover
	 */
	CDInfo content;
	/**
	 * backinlay, currently the only thing we want to print
	 */
	BackInlay* back;
	/**
	 * PDFhandle to write on
	 */
	PSDoc* sheet;
	/**
	 * Fonts used on this PS-document. pslib gives each font it generated a specific number
	 */
	FontSet  fonts;

	/**
	 * Create a fontset for this PS-Document
	 */
	void  loadfonts ();
	
};
#endif //CDCOVER_H

