/******************************************************************************
// Synopsis:
//-----------------------
// Dependencies:
//
//-----------------------
// CVS:
// $Author: ruschi $
// $Date: 2004/11/13 14:11:34 $
// $Revision: 1.3 $ 
//-----------------------------------------------------------------------------
//  Copyright (C) 2004 by Thomas Ruschival                               
//  ruschi@doomgarage.de                                                 
//                                                                       
//  This program is free software; you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation; either version 2 of the License, or    
//  (at your option) any later version.                                  
//                                                                       
//  This program is distributed in the hope that it will be useful,      
//  but WITHOUT ANY WARRANTY; without even the implied warranty of       
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
//  GNU General Public License for more details.                         
//                                                                       
//  You should have received a copy of the GNU General Public License    
//  along with this program; if not, write to the                        
//  Free Software Foundation, Inc.,                                      
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
*******************************************************************************/

#ifndef CDINFOFACTORY_H
#define CDINFOFACTORY_H
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <cctype>

#include <dlfcn.h>		/* needed for dynamic loading of Reader plugins */
#include <fnmatch.h>	/* Match the filename so we only try to load shared objects */
#include <sys/types.h>  
#include <unistd.h>
#include <dirent.h>

#include "cdinforeader.h"

#define FILENAME_FILTER "*.so"
#ifndef PLUGIN_PATH
#define PLUGIN_PATH "./readerplugins/"
#endif

using namespace std;
/**
 * Class CDInfoFactory
 * Generates a specific CDInfoReader then reads the content
 * of the source and returns the CDInfo depending on what
 * information is providied in infile
 */
class CDInfoFactory {
/**
 * Public stuff
 */
public:
	/**
	 * Constructors
	 */
	/**
	 * Constructor with container and a source set
	 * @param cdcontent Container for information to fill
	 * @param infile filename of source
	 */
	CDInfoFactory (CDInfo& cdcontent,string infile);

	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of content
	 * Container where the readers put their information
	 * @return the value of content
	 */
	CDInfo get_content ( );
		
	
	/**
	 * Set the value of content
	 * Container where the readers put their information
	 * @param value the value of content
	 */
	void set_content (CDInfo& value );
		
	
	/**
	 * Get the value of infile
	 * Location to to read it's not opened as a istram or stored as file because readers have to decide how to open and handle the Information
	 * @return the value of infile
	 */
	string get_source ( );
		
	
	/**
	 * Set the value of infile
	 * Location to to read it's not opened as a istram or stored as file because readers have to decide how to open and handle the Information
	 * @param value the value of infile
	 */
	void set_source (string value);
	
	/**
	 * Operations
	 */
	/**
	 * return a fitting reader for the specified infile
	 * getting Information from extension
	 * @param infile source to read
	 * @return cur_reader the suitable reader for this problem. or 0 if no reader is found
	 */
	CDInfoReader* getReader (string infile);
	
	/**
	 * content will be newly read from the specified source
	 */
	CDInfo  get_content (string infile);

		
/**
 * Private stuff
 */
private:
	/**
	 * Fields
	 */
	/**
	 * Container where the readers put their information
	 */
	 CDInfo content;
	/**
	 * Location to to read it's not opened as a istram or stored as file because readers have to decide how to open and handle the Information
	 */
	 string  source;
	/**
	 * Array of CDInfoReaders, that were dynamiccaly loaded.
	 */
	 vector<CDInfoReader*> readers;

	/**
	* Reader that can read the last source 
	*/
	 CDInfoReader* cur_reader;
	/**
	 * private operations 
	 */

	/**
	 * dynamically load specific CDInfoReader classes into the
	 * reader array.
	 */
	void  loadReaders ();


	
};
#endif //CDINFOFACTORY_H

