/******************************************************************************
* Synopsis:
*-----------------------
* Dependencies:
*
*-----------------------
* CVS:
* $Author: ruschi $
* $Date: 2004/11/11 21:11:20 $
* $Revision: 1.1.1.1 $ 
*-----------------------------------------------------------------------------
*  Copyright (C) 2004 by Thomas Ruschival                               
*  ruschi@doomgarage.de                                                 
*                                                                       
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or    
*  (at your option) any later version.                                  
*                                                                       
*  This program is distributed in the hope that it will be useful,      
*  but WITHOUT ANY WARRANTY; without even the implied warranty of       
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
*  GNU General Public License for more details.                         
*                                                                       
*  You should have received a copy of the GNU General Public License    
*  along with this program; if not, write to the                        
*  Free Software Foundation, Inc.,                                      
*  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
*******************************************************************************/
#include "cdinforeader_cdrom.h"
using namespace std;

CDInfoReader_cdrom::CDInfoReader_cdrom(string infile): CDInfoReader(infile){
	name = "Raw CDROM Reader";
	filetype = "any Blockdevice";
	description="Client to connect to CDDB \n";
	description+="Version: $Revision: 1.3 $ Copyright (c) 2004 Thomas Ruschival \n";
}

CDInfoReader_cdrom::CDInfoReader_cdrom(): CDInfoReader(){
	name = "Raw CDROM Reader";
	filetype = "any Blockdevice";
	description="Client to connect to CDDB \n";
	description+="Version: $Revision: 1.3 $ Copyright (c) 2004 Thomas Ruschival \n";
}


/**
* find out if the reader is suitable for the
* source
* @param infile string filename
* @return fits int 0 if false, 1 if it fits
*/
int CDInfoReader_cdrom::matches(string infile){
 	int result=0;
    struct stat buffer;
    result = stat(infile.c_str(),&buffer);

	if(S_ISBLK(buffer.st_mode)){
	  result =1;
	}	
	return result;
}


/**
 *
 */
void CDInfoReader_cdrom::readCDInfo(CDInfo &content){
  calcDiscID();
  lookup();
}


/**
*
*/
void CDInfoReader_cdrom::lookup(){
	 try{
	  ClientSocket client_socket("freedb.org", 888);
	  char reply[2048];
	  int i=0, pos=0, oldpos=0;
	  string str,read_command;
	  try{

		string hello="cddb hello Thomas Marvin Myripper 2.1\n";
		string proto="proto 6\n";
		string discid="cddb query " +discid_query + "\n";
		client_socket.sendMsg("\n");
		client_socket.readMsg(reply,2048);
		std::cout << "Start -- \n"<<  reply << std::endl;
		 
		client_socket.sendMsg(hello);
		client_socket.readMsg(reply,2048);
	    std::cout << hello << "-- \n"<<  reply << std::endl;
memset(reply,0,2048);
		client_socket.sendMsg(proto);
		client_socket.readMsg(reply,2048);
	    std::cout << proto << "-- \n"<<  reply << std::endl;
memset(reply,0,2048);
		client_socket.sendMsg(discid);
		client_socket.readMsg(reply,1024);
		str=reply;
		
		if(str.substr(0,3) == "200"){
			cout << " EXACT \n \n ";
			read_command = "cddb read ";
			oldpos = str.find_first_of(" ",0);
			pos = str.find_first_of(" ",oldpos+1);
			string categ =str.substr(oldpos+1,pos-oldpos-1);
			oldpos = pos+1;
			pos = str.find_first_of(" ",oldpos);
			string discid = str.substr(oldpos,pos-oldpos);
			read_command += categ + " " + discid + "\n";
			
		}
		else if(str.substr(0,3) == "210"){
			cout << " MORE EXACT MATCHES ";


		}
		client_socket.sendMsg(read_command);
memset(reply,0,2048);
	   
// 	  	while(client_socket.readLine(reply,2048,'\0') >0){
	  			
	  		    std::cout << "HELLO"<<  reply << std::endl;
		}	
	    memset(reply,0,2048);
	    client_socket.sendMsg("quit \n");
		client_socket.readMsg(reply,2048);
	    std::cout << "quit"<< "-- \n"<<  reply << std::endl;
	  }
	  
	  catch ( SocketException& e ) {
		std::cout << "Exception was caught:" << e.get_message() << "  -   " << e.get_cause() << " ERRNO: " << e.get_errcode() << "\n";
	  }
  }
  catch ( SocketException& e ) {
	  std::cout << "Exception was caught:" << e.get_message()<< "  -   " << e.get_cause() << "\n";
  }

}

/**
* Calculate the DISCID
* @return int 1 if sucessful, 0 otherwise
* 
*/
int CDInfoReader_cdrom::calcDiscID(){
  	tracks = readTOC();
    discid = cddbDiscID(tracks);
	cout << discid << endl;
    string str;
	int i;
	char buf[20];
    printf("---- >>> %08x %d \n \n ", discid, tracks);
	snprintf(buf, 20, "%08x", discid,tracks);
	discid_query=buf;
	itoa(str,tracks);
    discid_query += " " + str;

	for (i = 0; i < tracks; i++){
 		itoa(str, cdtoc[i].frame);
 		discid_query += " " + str;
 	}
 	itoa(str, (cdtoc[tracks].frame)/75);
    discid_query += " " + str;

cout << discid_query << endl;
    
    return 1;
}		

int CDInfoReader_cdrom::readTOC() {
    int drive = open(source.c_str(), O_RDONLY | O_NONBLOCK);
    struct cdrom_tochdr tochdr;
    struct cdrom_tocentry tocentry;
    int i;

    ioctl(drive, CDROMREADTOCHDR, &tochdr);
    for (i = tochdr.cdth_trk0; i <= tochdr.cdth_trk1; i++) {
        tocentry.cdte_track = i;
        tocentry.cdte_format = CDROM_MSF;
        ioctl(drive, CDROMREADTOCENTRY, &tocentry);
        cdtoc[i-1].min = tocentry.cdte_addr.msf.minute;
        cdtoc[i-1].sec = tocentry.cdte_addr.msf.second;
        cdtoc[i-1].frame = tocentry.cdte_addr.msf.frame;
        cdtoc[i-1].frame += cdtoc[i-1].min*60*75;
        cdtoc[i-1].frame += cdtoc[i-1].sec*75;
    }
    tocentry.cdte_track = 0xAA;
    tocentry.cdte_format = CDROM_MSF;
    ioctl(drive, CDROMREADTOCENTRY, &tocentry);
    cdtoc[tochdr.cdth_trk1].min = tocentry.cdte_addr.msf.minute;
    cdtoc[tochdr.cdth_trk1].sec = tocentry.cdte_addr.msf.second;
    cdtoc[tochdr.cdth_trk1].frame = tocentry.cdte_addr.msf.frame;
    cdtoc[tochdr.cdth_trk1].frame += cdtoc[tochdr.cdth_trk1].min*60*75;
    cdtoc[tochdr.cdth_trk1].frame += cdtoc[tochdr.cdth_trk1].sec*75;
    close(drive);
    return tochdr.cdth_trk1;
}

unsigned int CDInfoReader_cdrom::cddbSum(int n) {
    unsigned int ret;
    ret = 0;
    while (n > 0) {
      ret += (n % 10);
      n /= 10;
    }
    return ret;
}

unsigned long CDInfoReader_cdrom::cddbDiscID(int tot_trks) {
    int i, t = 0, n = 0;

    i = 0;
    while (i < tot_trks) {
      n = n + cddbSum((cdtoc[i].min * 60) + cdtoc[i].sec);
      i++;
    }
    t = ((cdtoc[tot_trks].min * 60) + cdtoc[tot_trks].sec) -
      ((cdtoc[0].min * 60) + cdtoc[0].sec);
    return ((n % 0xff) << 24 | t << 8 | tot_trks);
}


/**
 * Constructors/Destructors
 */
extern "C" void destroy(CDInfoReader* todestroy){
	delete todestroy;
}

extern "C" CDInfoReader* create(string infile){
	return new CDInfoReader_cdrom(infile);
}

